﻿# S2D-DiskInformation
# Quick S2D Disk Information Script

# Also support regular Storage Spaces

# Version 1.09 31-08-22
# Andrew Sharrad

param ([switch]$Quiet=$false,[string]$Filename)

$Version = "1.09"
$Date = "31/08/22"
$Author = "Andrew Sharrad"

$maxcountertimeoutinseconds=8

#$Global:ProgressPreference = 'SilentlyContinue' # stop progress popups

$ErrorActionPreference= 'silentlycontinue' # turn this off for debugging
#$ErrorActionPreference= 'continue'	# turn this on for debugging

$WarningPreference = 'SilentlyContinue' # turn this off for debugging
#$WarningPreference = 'Continue' # turn this on for debugging


# Get the ID and security principal of the current user account
$myWindowsID=[System.Security.Principal.WindowsIdentity]::GetCurrent()
$myWindowsPrincipal=new-object System.Security.Principal.WindowsPrincipal($myWindowsID)
 
# Get the security principal for the Administrator role
$adminRole=[System.Security.Principal.WindowsBuiltInRole]::Administrator
# Check to see if we are currently running "as Administrator"

if ($myWindowsPrincipal.IsInRole($adminRole))
    {
    # We are running "as Administrator" - so change the title and background color to indicate this
    #$Host.UI.RawUI.WindowTitle = $myInvocation.MyCommand.Definition + " S2D-DiskInformation " + $Version + " (Elevated)"
    #$Host.UI.RawUI.BackgroundColor = "DarkBlue"
    #clear-host
    } else {
    # We are not running "as Administrator" - so relaunch as administrator
   
    # Create a new process object that starts PowerShell
    $newProcess = new-object System.Diagnostics.ProcessStartInfo "PowerShell";
   
    # Specify the current script path and name as a parameter
    #$newProcess.Arguments = $myInvocation.MyCommand.Definition;

		$newProcess.Arguments = "& '" + $script:MyInvocation.MyCommand.Path + "'"  

    # Indicate that the process should be elevated
    $newProcess.Verb = "runas";
   
    # Start the new process
	write-Host "S2D-DiskInformation: $Version $Author $Date" -foreground green
	Write-Host "Process Started: $runtime" -foreground green
    Write-Host "Administrator rights are not present - spawning a second copy with elevated Administrator rights" -Foreground cyan
    try {
    $null = [System.Diagnostics.Process]::Start($newProcess);
    }
   
    catch {
    Write-Host "PROCESS FAILURE: Administrator rights are not present - attempt to spawn elevated process failed - S2D-DiskInformation Terminated" -Foreground Magenta
    }

	Write-Host "=======================================================================================================================" -foreground white
	
    # Exit from the current, unelevated, process
    exit
    }


if ($quiet -eq $false) {
    write-Host "S2D-DiskInformation: $Version $Author $Date - Getting Disk Information" -foreground green
}

	
If (-NOT ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole(`

    [Security.Principal.WindowsBuiltInRole] "Administrator"))

{
    Write-Host "PROCESS FAILURE: Administrator rights are not present - S2D-DiskInformation Terminated" -Foreground Magenta
    
#	Start-Sleep -s 10

	read-host “Press ENTER to exit...”
	
    Break
}
# this code is still included in case of self-elevation issue

$pooleddisk = @()

$pools = get-storagepool *

foreach ($pool in $pools) {

	$pooldisks=get-storagepool -FriendlyName $pool.FriendlyName | get-physicaldisk

	foreach ($pooldisk in $pooldisks) {

		if ($pooldisk.CanPool -eq $true -or $pool.FriendlyName -ne "Primordial") {
			$pooleddisk += [PSCustomObject]@{
				'PoolFriendlyName' = $pool.FriendlyName
				'PoolOperationalStatus' = $pool.OperationalStatus
				'PoolHealthStatus' =  $pool.HealthStatus
				'PoolIsPrimordial' = $pool.IsPrimordial
				'PoolIsReadOnly' = $pool.IsReadonly
				'DeviceID' = $pooldisk.DeviceID
				'SN' = $pooldisk.SerialNumber
				'FriendlyName' = $pooldisk.FriendlyName
				'UniqueID' = $pooldisk.UniqueID
				'CanPool' = $pooldisk.Canpool
			}
			}
	}
}


$disks = get-physicaldisk

$output = @()


$GetCounterInfo = {
    Param ($disk)
    $counters = $disk | get-storagereliabilitycounter -ErrorAction Silentlycontinue
    return $counters
} 
$disksprocessed=0
$i=0
if ($quiet -eq $false) {
	Write-Progress -Activity "Getting Disk Information in Progress" -Status "$i% Complete" -PercentComplete $i;
}
foreach ($disk in $disks) {
    $disksprocessed++
    $i = [math]::round(($disksprocessed / $disks.count)*100,2)
    #Write-host "Getting Information on Disk $i of $($disks.count)"
    $Job = Start-Job -Name "Disk $($disk.Number)" -ScriptBlock $GetCounterInfo -ArgumentList $disk
    $NodeObj = $disk | Get-StorageNode -PhysicallyConnected
    if ($NodeObj -eq $null) {
	    $NodeName = '-'
	} else {
		$NodeName = $NodeObj.Name.Split(".")[0]
	}

	if ($disk.SlotNumber -ne $NULL) {
		$SlotString = [int]$disk.SlotNumber
	} else {
		$SlotString = $null
	}

    $gotcounters=$false

    for ($a=0; ($a -lt $maxcountertimeoutinseconds -and $gotcounters -eq $false) ; $a++) {
        Wait-Job -Timeout 1 -Job $Job | Out-Null

        if ($Job.State -ne 'Running') {
            $gotcounters=$true
        }
    }

    if ($Job.State -eq 'Running') {
		Stop-Job -Job $Job
        $gotcounters=$false
	} else {
        $gotcounters=$true
    }
	$counters = Receive-Job -Job $Job
    Remove-Job -Job $Job

    $errorcount = [int]$counters.ReadErrorsUnCorrected + $counters.WriteErrorsUnCorrected + $counters.ReadErrorsCorrected + $counters.WriteErrorsCorrected
    $size = [math]::round($disk.Size/1024/1024/1024/1024,2)


	$matchedpooleddisk = @($pooleddisk | where { $_.SN -eq $disk.SerialNumber -and $_.DeviceID -eq $disk.DeviceID -and $_.UniqueID -eq $disk.UniqueID -and $_.FriendlyName -eq $disk.FriendlyName } | Select -First 1)

	if ($matchedpooleddisk.count -eq 0) {
	$matchedpooleddisk = @()
	$matchedpooleddisk += [PSCustomObject]@{
		'PoolFriendlyName' = "(not in a pool)"
		'PoolOperationalStatus' = ""
		'PoolHealthStatus' = ""
		'PoolIsPrimordial' = ""
		'PoolIsReadOnly' = " "
		}
		
	}

    $output += [PSCustomObject]@{
        'Node' = $NodeName
        'DeviceID' = $disk.DeviceID
        'Slot' = $SlotString
        'SizeTB' = $size
        'FriendlyName' = $disk.FriendlyName
        'SerialNumber' = $disk.SerialNumber
		'AdapterSerialNumber' = $disk.AdapterSerialNumber
		'FirmwareVersion' = $disk.FirmwareVersion
        'OperationalStatus' = $disk.OperationalStatus
        'HealthStatus' = $disk.HealthStatus
		'PoolFriendlyName' = $matchedpooleddisk.PoolFriendlyName
        'GotCounterInformation' = $gotcounters
        'CounterTime' = $a
        'ErrorCount' = $errorcount
        'PowerOnHours' = $counters.PowerOnHours
        'LoadUnloadCount' = $counters.LoadUnloadCycleCount
	    'LoadUnloadCountMax' = $counters.LoadUnloadCycleCountMax
        'Location' = $disk.PhysicalLocation
        'BusType' = $disk.BusType
        'MediaType' = $disk.MediaType
        'CanPool' = $disk.CanPool
        'CannotPoolReason' = $disk.CannotPoolReason
        'Usage' = $disk.Usage
		'StartStopCount' = $counters.StartStopCycleCount
		'ReadErrorsCorrected' = $counters.ReadErrorsCorrected
		'ReadErrorsUnCorrected' = $counters.ReadErrorsUnCorrected
		'ReadErrorsTotal' = $counters.ReadErrorsTotal
		'ReadLatencyMax' = $counters.ReadLatencyMax
		'WriteErrorsCorrected' = $counters.WriteErrorsCorrected
		'WriteErrorsUnCorrected' = $counters.WriteErrorsUnCorrected
		'WriteErrorsTotal' = $counters.WriteErrorsTotal
		'WriteLatencyMax' = $counters.WriteLatencyMax
		'FlushLatencyMax' = $counters.FlushLatencyMax
		'Temperature' = $counters.Temperature
		'TemperatureMax' = $counters.TemperatureMax
		'Wear' = $counters.Wear
		'ManufactureDate' = $counters.ManufactureDate
		'LogicalSectorSize' = $disk.LogicalSectorSize
		'PhysicalSectorSize' = $disk.PhysicalSectorSize
		'PoolOperationalStatus' = $matchedpooleddisk.PoolOperationalStatus
		'PoolHealthStatus' = $matchedpooleddisk.PoolHealthStatus
		'PoolIsPrimordial' = $matchedpooleddisk.PoolIsPrimordial
		'PoolIsReadOnly' = $matchedpooleddisk.PoolIsReadOnly
    }
    if ($quiet -eq $false) {
		Write-Progress -Activity "Getting Disk Information in Progress" -Status "$i% Complete" -PercentComplete $i;
	}
}

$output = $output | Sort Node, Slot, { [int]$_.DeviceID }


if ($quiet -eq $false) {
    $Selected = $Output | Out-GridView -Title 'Disks' -PassThru
} else {
	$Selected = $Output
}

if ($Filename) {
	$Selected | Export-CSV -Path $Filename -NoTypeInformation
} else {
    $Selected | fl *
	if ($quiet -eq $false) {
		write-host "Variables available: All Items: `$Output, Selected Items Only: `$Selected"
	}
}

if ($MyInvocation.InvocationName -eq '&' -and $quiet -eq $false) { # Called via Explorer Window
	read-host "Press Any Key to Continue"
} # else Called via Existing Prompt
